#define HISTORY_SIZE 20
#define SENSITIVITY 45000
#define COUNT_TIME  700

unsigned int frequency;
unsigned int history[HISTORY_SIZE];
unsigned char historyPos = 0;
unsigned long accum = 0;
long delta = 0;

/** \note Strange behavior: Soft_UART_Write drives also RC2 pin */
void dbg_write_str(const char* str) {
     while (*str) {
           Soft_UART_Write(*str);
           str++;
     }
}

void ltoa(long i, char* str) {
     int pos = 7;
     str[8] = '\0';
     while (pos >= 0) {
        unsigned char hex = i & 0x0F;
        if (hex < 10) {
           str[pos] = '0' + hex;
        } else {
          str[pos] = 'A' + hex - 10;
        }
        i >>= 4;
        pos--;
     }
}

void dbg_write_long(long val) {
     char buf[9];
     char* ptr;
     ltoa(val, buf);
     ptr = buf;
     while (*ptr) {
           Soft_UART_Write(*ptr);
           ptr++;
     }
}

void beepH(void) {
     int i;
     for (i=0; i<20; i++) {
           PORTA.B4 = 1;
           Delay_ms(1);
           PORTA.B4 = 0;
           Delay_ms(1);
     }
}

void beepL(void) {
     int i;
     for (i=0; i<20; i++) {
           PORTA.B4 = 1;
           Delay_ms(2);
           PORTA.B4 = 0;
           Delay_ms(2);
     }
}

void measureFrequency(void) {
    TMR1H = 0x00;
    TMR1L = 0x00;
    T1CON = 0b00000111;   // Enable Timer 1
    Delay_ms(COUNT_TIME);
    T1CON = 0b00000110;   // Disable Timer 1
    frequency = TMR1H;
    frequency = frequency << 8;
    frequency = frequency + TMR1L;
}

void main() {
     int i;
     
     /* Set the internal clock to 8 Mhz */
     //OSCCON = 0x70;

     /* Set the internal clock to 1 MHz */
     OSCCON = 0x40;

     ANSEL = 0x00;

     ANSEL.B7 = 0;
#if 0
     TRISC.B3 = 0; //UART output
     if (Soft_UART_Init(&PORTC, 5 /*RX*/, 3 /*TX*/, 9600, 0)) {
        /** \note will fail with low internal clock speed, increase it before enabling */
        for(;;) {
        }
     }
#endif

     /* R-0 R-0 R/W-0 R/W-0 R/W-0 R/W-0 R/W-0 R/W-0
        C2OUT C1OUT C2INV C1INV CIS CM2 CM1 CM0
        only in this mode comparator 2 output is active */
     CMCON0 = 0x06;

     ANSEL.B4 = 1;
     ANSEL.B5 = 1;

     TRISC.B4 = 0;           // comparator 2: output

     TRISA.B4 = 0;           // headphone: output

     beepH();
     beepL();
     beepH();
     beepL();

     measureFrequency();
     for (i=0; i<HISTORY_SIZE; i++) {
         history[i] = frequency;
     }

     do {
        measureFrequency();
#if 0
        dbg_write_str("Cnt: ");
        dbg_write_long(frequency);
        dbg_write_str("\r\n");
#endif
        accum = 0;
        for (i=0; i<HISTORY_SIZE; i++) {
            accum += history[i];
        }
        
        delta = (long)frequency * HISTORY_SIZE;

        if (delta > accum) {
           delta = delta - accum;
           delta *= SENSITIVITY;
           if (delta > accum) {
              beepH();
           }
        } else {
          delta = accum - delta;
           delta *= SENSITIVITY;
           if (delta > accum) {
              beepL();
           }
        }

        history[historyPos] = frequency;
        historyPos++;
        if (historyPos >= HISTORY_SIZE) {
           historyPos = 0;
        }

     } while(1);          // Endless loop
}